/*
 *  GnomeZip
 *  actions.c
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _ACTIONS_C_
#define _ACTIONS_C_

#include <sys/stat.h>
#include <unistd.h>
#include <actions.h>
#include <misc.h>
#include <libgarp/garp-archive.h>
#include <libgarp/garp-archive-entry.h>
#include <gnome.h>
#include <callbacks.h>
#include <overwrite-dialog.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

static GList * show_overwrite_dlg (MainApp *app, GList *list,
  const gchar *extract_to_dir, gboolean select_all);

/*********  Please keep function names in alphabetic order!  **********/


static void
dialog_button_click (GtkObject *widget, gpointer user_data)
{
    gint *x;
    GtkWidget *dialog;

    x = (gint *) gtk_object_get_data (widget, "x");
    dialog = (GtkWidget *) gtk_object_get_data (widget, "dialog");
    *x = GPOINTER_TO_INT (gtk_object_get_data (widget, "i"));
    gtk_widget_hide (dialog);
}


static void
free_list (GList **list, void (*free_cb)(void *p))
{
    guint i;

    if (g_list_length (*list) > 0)
    for (i = g_list_length (*list) - 1; i >= 0; i--)
    {
        gpointer entry;

        entry = g_list_nth_data (*list, i);
        *list = g_list_remove (*list, entry);
        (*free_cb) (entry);
        if (i == 0) break;
    }
}


void
gz_close_archive (MainApp *app)
{
    if (app->clist != NULL && GTK_CLIST (app->clist)->rows > 0)
    {
        guint i;

        for (i = 0; i < GTK_CLIST (app->clist)->rows; i++)
        {
            GarpArchiveEntry *entry = (GarpArchiveEntry *) \
              gtk_clist_get_row_data (GTK_CLIST (app->clist), i);

            garp_archive_entry_destroy (entry);
        }

        gtk_clist_clear (GTK_CLIST (app->clist));
    }

    if (app->archive != NULL) garp_archive_destroy (app->archive);
    gtk_window_set_title (GTK_WINDOW (app->window), _("GnomeZip"));
}


gboolean
gz_extract_archive (MainApp *app, const gchar *extract_to_dir)
{
    guint i, result = 0;
    guint count;
    GList *list, *list2;

    list = gtk_clist_get_selections (GTK_CLIST (app->clist));
    count = g_list_length (list);

    if (count == 0)
    {
        g_list_free (list);
        list = NULL;
        for (i = 0; i < g_list_length (GTK_CLIST (app->clist)->row_list); i++)
            list = g_list_append (list, gtk_clist_get_row_data (GTK_CLIST (
              app->clist), i));

        list2 = show_overwrite_dlg (app, list, extract_to_dir, TRUE);
        if (list2 != NULL)
        {
            result = garp_archive_extract_files (app->archive, list2, (gchar *) extract_to_dir,
              GARP_ARCHIVE_OVERWRITE_ACTION_ALL, NULL, NULL, NULL);
            /* result = garp_archive_extract_all (app->archive, (gchar *) extract_to_dir,
              GARP_ARCHIVE_OVERWRITE_ACTION_ALL, NULL, NULL, NULL); */
            g_list_free (list2);
        }
    } else
    {
        g_list_free (list);
        list = NULL;
        for (i = 0; i < g_list_length (GTK_CLIST (app->clist)->row_list); i++)
        {
            GtkCListRow *row = g_list_nth_data (GTK_CLIST (app->clist)->row_list, i);
            if (row->state == GTK_STATE_SELECTED)
            {
                GarpArchiveEntry *entry;

                entry = (GarpArchiveEntry *) gtk_clist_get_row_data (
                  GTK_CLIST (app->clist), i);
                list = g_list_append (list, entry);
            }
        }

        list2 = show_overwrite_dlg (app, list, extract_to_dir, TRUE);
        if (list2 != NULL)
        {
            result = garp_archive_extract_files (app->archive, list2, (gchar *) extract_to_dir,
              GARP_ARCHIVE_OVERWRITE_ACTION_ALL, NULL, NULL, NULL);
            g_list_free (list2);
        }
    }

    g_list_free (list);
    if (result == 0)
        return TRUE;
    else
        return FALSE;
}


gboolean
gz_open_archive (MainApp *app, const gchar *filename)
{
    GList *list = NULL;
    gchar **titles, *wintitle;
    GarpArchive *arch, *arch2;
    GarpArchiveListResult result;
    guint i;

    arch = app->archive;
    app->archive = garp_archive_open (filename);
    if (app->archive == NULL)
    {
        app->archive = arch;
        return FALSE;
    }
    while (gtk_events_pending ()) gtk_main_iteration ();
    result = garp_archive_list (app->archive, &list);
    while (gtk_events_pending ()) gtk_main_iteration ();
    arch2 = app->archive;
    app->archive = arch;

    if (result == GARP_ARCHIVE_LIST_INVALID_ARCHIVE)
    {
        gnome_error_dialog (_("This archive is damaged.\nIf you downloaded"
          "this file from the Internet, try to download it again."));
        free_list (&list, (void (*)(void *p)) garp_archive_entry_destroy);
        return FALSE;
    } else if (result == GARP_ARCHIVE_LIST_INVALID_ENTRY)
    {
        gchar *msg, *fn;

        fn = GARP_ARCHIVE_ENTRY (g_list_nth_data (list, g_list_length (list)
          - 1))->filename;
        msg = g_strdup_printf (_("One of the entries in this archive is damaged.\n"
          "The last intact entry is: %s"), fn);
        gnome_error_dialog (msg);
        g_free (msg);
        free_list (&list, (void (*)(void *p)) garp_archive_entry_destroy);
        return FALSE;
    } else if (result == GARP_ARCHIVE_LIST_UNKNOWN_ERROR)
    {
        gnome_error_dialog (_("An unknown error occured. This could be\n"
          "a bug. If you can reproduce this error, please send a bugreport\n"
          "to one of the authors."));
        free_list (&list, (void (*)(void *p)) garp_archive_entry_destroy);
        return FALSE;
    }

    gz_close_archive (app);

    app->archive = arch2;
    wintitle = g_strdup_printf ("%s - %s", _("GnomeZip"), filename);
    gtk_window_set_title (GTK_WINDOW (app->window), wintitle);
    g_free (wintitle);

    /* Create a new clist with the new titles */
    if (app->clist != NULL) gtk_widget_destroy (app->clist);
    titles = (gchar **) g_new0 (gchar **, app->archive->field_count + 2);
    titles[0] = "#";
    for (i = 1; i < app->archive->field_count + 1; i++)
        titles[i] = g_strdup (app->archive->fields[i - 1].field_name);

    app->clist = gtk_clist_new_with_titles (app->archive->field_count + 1, titles);
    gtk_clist_set_selection_mode (GTK_CLIST (app->clist), GTK_SELECTION_EXTENDED);
    gtk_signal_connect (GTK_OBJECT (app->clist), "button-press-event",
      GTK_SIGNAL_FUNC (mainapp_clist_button_event), app);
    gtk_signal_connect (GTK_OBJECT (app->clist), "button-release-event",
      GTK_SIGNAL_FUNC (mainapp_clist_button_event), app);
    gtk_signal_connect (GTK_OBJECT (app->clist), "key-press-event",
      GTK_SIGNAL_FUNC (mainapp_clist_key_event), app);
    gtk_signal_connect (GTK_OBJECT (app->clist), "key-release-event",
      GTK_SIGNAL_FUNC (mainapp_clist_key_event), app);
    gtk_signal_connect (GTK_OBJECT (app->clist), "click_column",
      GTK_SIGNAL_FUNC (mainapp_clist_click_column), NULL);
    gtk_widget_show (app->clist);
    gtk_container_add (GTK_CONTAINER (glade_lookup_widget (app->window,
      "scrollwindow")), app->clist);


    while (gtk_events_pending ()) gtk_main_iteration ();
    gtk_clist_freeze (GTK_CLIST (app->clist));
    for (i = 0; i < g_list_length (list); i++)
    {
        gchar **values = (gchar **) g_new0 (gchar **,
          app->archive->field_count + 2);
        GarpArchiveEntry *entry = (GarpArchiveEntry *) g_list_nth_data (list,
          i);
        guint x;

        values[0] = g_strdup_printf ("%d", i + 1);
        for (x = 1; x < app->archive->field_count; x++)
        {
            gchar *value;

            value = (gchar *) g_list_nth_data (entry->values,
              x - 1);
            values[x] = g_strdup (value);
        }

        gtk_clist_append (GTK_CLIST (app->clist), values);
        gtk_clist_set_row_data (GTK_CLIST (app->clist), i, entry);
        while (gtk_events_pending ()) gtk_main_iteration ();
    }

    for (i = 0; i < app->archive->field_count + 1; i++)
        gtk_clist_set_column_width (GTK_CLIST (app->clist), i,
          gtk_clist_optimal_column_width (GTK_CLIST (app->clist), i));
    gtk_clist_thaw (GTK_CLIST (app->clist));
    while (gtk_events_pending ()) gtk_main_iteration ();

    {
        GtkMenu *menu = app->sort_menu;
        GtkWidget *item;

        free_list (&app->sort_items, (void (*)(void *p)) gtk_widget_destroy);

        item = gtk_menu_item_new_with_label (_("Number"));
        gtk_menu_append (menu, item);
        app->sort_items = g_list_append (app->sort_items, item);
        gtk_tooltips_set_tip (app->tooltips, item,
          _("The number of this entry"), "");
        gtk_signal_connect (GTK_OBJECT (item), "activate",
          GTK_SIGNAL_FUNC (mainapp_clist_click_column), GINT_TO_POINTER (0));

        for (i = 1; i < app->archive->field_count + 1; i++)
        {
            item = gtk_menu_item_new_with_label (
              g_strdup (app->archive->fields[i - 1].field_name)
            );
            gtk_menu_append (menu, item);
            app->sort_items = g_list_append (app->sort_items, item);
            gtk_tooltips_set_tip (app->tooltips, item,
              app->archive->fields[i - 1].field_description, "");
            gtk_signal_connect (GTK_OBJECT (item), "activate",
              GTK_SIGNAL_FUNC (mainapp_clist_click_column), GINT_TO_POINTER (i));
        }
        gtk_widget_show_all (GTK_WIDGET (menu));
    }

    main_app_status (app, NULL);
    g_list_free (list);
    gtk_widget_grab_focus (app->clist);
    return TRUE;
}


static GList *
show_overwrite_dlg (MainApp *app, GList *list, const gchar *extract_to_dir,
  gboolean select_all)
{
    OverwriteDialog *dialog;
    GList *result = NULL;
    gint i, x = -1;

    dialog = app->overwrite_dialog;
    dialog->extract_to_dir = (gchar *) extract_to_dir;
    gtk_clist_clear (GTK_CLIST (dialog->clist));
    gtk_clist_freeze (GTK_CLIST (dialog->clist));

    for (i = 0; i < g_list_length (list); i++)
    {
        GarpArchiveEntry *entry;
        gchar *filename, *path;
        struct stat buf;

        entry = (GarpArchiveEntry *) g_list_nth_data (list, i);
        filename = g_concat_path (entry->filename, entry->pathname);

        path = g_strdup_printf ("%s%s", extract_to_dir, filename);
        if (stat (path, &buf) == 0)
        {
            gchar *field[4] = {"#", NULL, "?", NULL};
            GtkCListRow *row;
            GtkWidget *widget;

            field[0] = g_strdup_printf ("%d", x + 1);
            field[1] = g_strdup (filename);
            field[2] = g_strdup_printf ("%d bytes", (int) buf.st_size);

            x = gtk_clist_append (GTK_CLIST (dialog->clist), field);
            gtk_clist_set_row_data (GTK_CLIST (dialog->clist), x, entry);

            widget = gtk_check_button_new ();
            if (select_all == TRUE)
                gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget),
                  TRUE);
            row = g_list_nth_data (GTK_CLIST (dialog->clist)->row_list, x);

            /* row->cell->u.widget = widget;
            GTK_CLIST_CLASS (GTK_OBJECT (dialog->clist)->klass)->set_cell_contents
              (GTK_CLIST (dialog->clist), row, 1, GTK_CELL_WIDGET, NULL, 0,
              NULL, NULL); */
        } else
            result = g_list_append (result, entry);
        g_free (path);
    }
    gtk_clist_thaw (GTK_CLIST (dialog->clist));

    if (g_list_length (GTK_CLIST (dialog->clist)->row_list) > 0)
    {
        GList *signals = NULL;

        if (select_all == TRUE)
            gtk_clist_select_all (GTK_CLIST (dialog->clist));
        gtk_clist_set_column_width (GTK_CLIST (dialog->clist),
          0, gtk_clist_optimal_column_width (GTK_CLIST (dialog->clist), 0));
        gtk_clist_set_column_min_width (GTK_CLIST (dialog->clist),
          1, gtk_clist_optimal_column_width (GTK_CLIST (dialog->clist), 1));
        gtk_clist_set_column_width (GTK_CLIST (dialog->clist),
          2, gtk_clist_optimal_column_width (GTK_CLIST (dialog->clist), 2));

        /* i = gnome_dialog_run (GNOME_DIALOG (dialog->window)); */
        for (i = 0; i < g_list_length (GNOME_DIALOG (dialog->window)->buttons); i++)
        {
            GtkObject *button;

            button = GTK_OBJECT (g_list_nth_data (GNOME_DIALOG (
              dialog->window)->buttons, i));
            gtk_object_set_data (button, "x", &x);
            gtk_object_set_data (button, "i", GINT_TO_POINTER (i));
            gtk_object_set_data (button, "dialog", dialog->window);
            signals = g_list_append (signals,
              GINT_TO_POINTER (gtk_signal_connect (button, "clicked",
              GTK_SIGNAL_FUNC (dialog_button_click), NULL))
            );
        }

        gtk_widget_run (dialog->window);
        for (i = 0; i < g_list_length (signals); i++)
            gtk_signal_disconnect (GTK_OBJECT (
              g_list_nth_data (GNOME_DIALOG (dialog->window)->buttons, i)),
              GPOINTER_TO_INT (g_list_nth_data (signals, i))
            );
        g_list_free (signals);

        i = x;
        if (i != 0) return result;

        for (i = 0; i < g_list_length (GTK_CLIST (dialog->clist)->row_list); i++)
        {
            GtkCListRow *row = g_list_nth_data (GTK_CLIST (dialog->clist)->row_list, i);
            if (row->state == GTK_STATE_SELECTED)
                result = g_list_append (result, gtk_clist_get_row_data (GTK_CLIST (
                  dialog->clist), i));
        }
    }

    return result;
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _ACTIONS_C_