/*
 *  GnomeZip
 *  callbacks.c: callbacks for the main window
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _CALLBACKS_C_
#define _CALLBACKS_C_

#include <gnome.h>
#include <gnomezip.h>
#include <callbacks.h>
#include <misc.h>
#include <config.h>
#include <actions.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


/*********  Please keep function names in alphabetic order!  **********/


void
main_app_about_clicked (GtkMenuItem *menuitem, gpointer user_data)
{
    const gchar *authors[] = {
      "Hongli Lai <hongli@telekabel.nl",
      NULL
    };
    GtkWidget *about = gnome_about_new (
      N_(PROGNAME),
      VERSION,
      N_("Copyright (c) 2000 Hongli Lai"),
      (const gchar **) authors,
      N_("An archiving program for Gnome."),
      NULL
    );
    gtk_widget_show (about);
}


void
main_app_close (GtkWidget *widget, gpointer user_data)
{
    gz_close_parent_window (widget, "mainapp");
}


void
main_app_exit (GnomeApp *app, gpointer user_data)
{
    gint i;
    MainApp *mainapp;

    for (i = g_list_length (winlist) - 1; i >= 0; i--)
    {
        mainapp = g_list_nth_data (winlist, i);
        gtk_widget_destroy (mainapp->window);
    }
}


void
main_app_new_win (GnomeApp *app, gpointer user_data)
{
    gtk_widget_show (main_app_new ()->window);
}


void
main_app_on_destroy (GnomeApp *app, gpointer user_data)
{
    main_app_destroy (MAIN_APP (gtk_object_get_data (GTK_OBJECT (app), "APP_STRUCT")));
}


void
main_app_status (MainApp *app, const gchar *statusmsg)
{
    if (statusmsg != NULL)
        gnome_appbar_set_status (GNOME_APPBAR (GNOME_APP (app->window)->statusbar),
          statusmsg);
    else if (app->archive != NULL)
    {
        guint i, count = 0;

        for (i = 0; i < g_list_length (GTK_CLIST (app->clist)->row_list); i++)
        {
            GtkCListRow *row = g_list_nth_data (GTK_CLIST (
              app->clist)->row_list, i);
            if (row->state == GTK_STATE_SELECTED)
                count++;
        }

        gnome_appbar_set_status (GNOME_APPBAR (GNOME_APP (app->window)->statusbar),
          g_strdup_printf (_("%d files in archive, %d items selected"), i, count));
    }
}


void
mainapp_clist_button_event (GtkWidget *widget, GdkEventButton *event,
  gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    main_app_status (app, NULL);
}


void
mainapp_clist_click_column (GtkWidget *widget, gint column, gpointer gdata)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    GtkCList *clist = GTK_CLIST (app->clist);

    if (column != clist->sort_column)
        gtk_clist_set_sort_column (clist, column);
    else {
        if (clist->sort_type == GTK_SORT_ASCENDING)
            clist->sort_type = GTK_SORT_DESCENDING;
        else
            clist->sort_type = GTK_SORT_ASCENDING;
    }
    gtk_clist_sort (clist);
}


void
mainapp_clist_key_event (GtkWidget *widget, GdkEventKey *event,
  gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    main_app_status (app, NULL);
}


void
mainapp_clist_invert_selection (GtkWidget *widget, gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    GList *sels;
    guint i;

    sels = GTK_CLIST (app->clist)->row_list;
    for (i = 0; i < g_list_length (sels); i++)
    {
        GtkCListRow *row = g_list_nth_data (sels, i);
        if (row->state == GTK_STATE_SELECTED)
            gtk_clist_unselect_row (GTK_CLIST (app->clist), i, 0);
        else if (row->state == GTK_STATE_NORMAL)
            gtk_clist_select_row (GTK_CLIST (app->clist), i, 0);
    }
}


void
mainapp_clist_select_all (GtkWidget *widget, gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    gtk_clist_select_all (GTK_CLIST (app->clist));
}


void
mainapp_clist_unselect_all (GtkWidget *widget, gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    gtk_clist_unselect_all (GTK_CLIST (app->clist));
}


void
mainapp_extract1_clicked (GtkWidget *widget, gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");
    gchar *title;
    guint i, count = 0;

    if (app->archive == NULL) return;

    for (i = 0; i < g_list_length (GTK_CLIST (app->clist)->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (GTK_CLIST (app->clist)->row_list, i);
        if (row->state == GTK_STATE_SELECTED)
            count++;
    }

    if (count == 0)
        title = _("Extract All Files");
    else if (count == 1)
        title = _("Extract File");
    else
        title = _("Extract Files");

    gtk_window_set_title (GTK_WINDOW (app->extract_filesel), title);
    if (gz_file_sel_run_and_close (app->extract_filesel) == TRUE)
        gz_extract_archive (app, app->extract_filesel->filename);
}


void
mainapp_open1_clicked (GtkWidget *widget, gpointer user_data)
{
    MainApp *app = gtk_object_get_data (GTK_OBJECT (glade_lookup_widget (
      GTK_WIDGET (widget), "mainapp")), "APP_STRUCT");

    if (gz_file_sel_run_and_close (app->open_filesel) == TRUE)
    {
        guint i;

        if (check_file (app->open_filesel->filename) == TRUE &&
            gz_open_archive (app, app->open_filesel->filename) == FALSE)
        {
            gchar *msg = g_strdup_printf (_("Error while opening file:\n'%s'\n"
              "This file format is not supported."),
              app->open_filesel->filename);

            gnome_error_dialog (msg);
            g_free (msg);
        }

        if (g_list_length (app->open_filesel->files) > 1)
        for (i = 1; i < g_list_length (app->open_filesel->files); i++)
        {
            gchar *filename = (gchar *) g_list_nth_data (app->open_filesel->files, i);
            MainApp *newapp;

            if (check_file (filename) == FALSE)
                continue;

            newapp = main_app_new ();
            while (gtk_events_pending ()) gtk_main_iteration ();
            if (gz_open_archive (newapp, filename) == FALSE)
            {
                gchar *msg = g_strdup_printf (_("Error while opening file:\n'%s'"), filename);

                gnome_error_dialog (msg);
                gtk_widget_destroy (newapp->window);
                g_free (msg);
                break;
            } else
                gtk_widget_show (newapp->window);
            while (gtk_events_pending ()) gtk_main_iteration ();
        }
    }
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _CALLBACKS_C_