/*
 *  GnomeZip
 *  filesel.h: GnomeZip's file selector
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _FILESEL_H_
#define _FILESEL_H_

#include <gtk/gtk.h>
#include <glib.h>
#include <glade/glade-xml.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GZ_TYPE_FILE_SEL            (gz_file_sel_get_type ())
#define GZ_FILE_SEL(obj)            GTK_CHECK_CAST (obj, GZ_TYPE_FILE_SEL, GzFileSel)
#define GZ_FILE_SEL_CLASS(klass)    GTK_CHECK_CLASS_CAST ((klass), GZ_TYPE_FILE_SEL, GzFileSelClass)
#define GZ_IS_FILE_SEL(obj)         GTK_CHECK_TYPE (obj, GZ_TYPE_FILE_SEL)
#define GZ_IS_FILE_SEL_CLASS(klass) GTK_CHECK_TYPE ((klass), GZ_TYPE_FILE_SEL)

typedef enum
{
    GZ_FILE_SEL_TYPE_FILE_ONLY,
    GZ_FILE_SEL_TYPE_DIR_ONLY,
    GZ_FILE_SEL_TYPE_FILE_AND_DIR
} GzFileSelType;

typedef struct _GzFileSel
{
    GtkFileSelection parent_struct;
    gboolean multi_file_select, multi_dir_select;
    GzFileSelType selection_type;

    GList *files;
    gchar *filename;
} GzFileSel;

typedef struct _GzFileSelClass
{
    GtkFileSelectionClass parent_klass;
    void (*select) (GzFileSel *filesel, gchar *selected_file, GList *selected_files,
      gpointer user_data);
    void (*cancel) (GzFileSel *filesel, gpointer user_data);
} GzFileSelClass;


guint gz_file_sel_get_type (void);
GzFileSel *gz_file_sel_new (const gchar *title);

gchar *gz_file_sel_get_filename (GzFileSel *filesel);
gboolean gz_file_sel_get_muli_dir_select (GzFileSel *filesel);
gboolean gz_file_sel_get_muli_file_select (GzFileSel *filesel);
GzFileSelType gz_file_sel_get_selection_type (GzFileSel *filesel);
void gz_file_sel_set_multi_dir_select (GzFileSel *filesel, gboolean value);
void gz_file_sel_set_multi_file_select (GzFileSel *filesel, gboolean value);
gboolean gz_file_sel_run_and_close (GzFileSel *filesel);
void gz_file_sel_set_selection_type (GzFileSel *filesel, GzFileSelType selection_type);


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _FILESEL_H_