/*
 *  GnomeZip
 *  gnomezip.c: GnomeZip's main window
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNOMEZIP_C_
#define _GNOMEZIP_C_

#include <gnome.h>
#include <gnomezip.h>
#include <misc.h>
#include <callbacks.h>
#include <actions.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


GList *winlist;


static void
main_app_init (MainApp *app)
{
    GtkWidget *tearoff;

    winlist = g_list_append (winlist, app);
    app->clist = NULL;
    app->sort_items = NULL;

    while (gtk_events_pending ()) gtk_main_iteration ();
    app->xml_data = glade_xml_new (gz_get_glade_file ("gnomezip.glade"), NULL);
    while (gtk_events_pending ()) gtk_main_iteration ();
    glade_xml_signal_autoconnect (app->xml_data);

    while (gtk_events_pending ()) gtk_main_iteration ();
    app->window = glade_xml_get_widget (app->xml_data, "mainapp");
    while (gtk_events_pending ()) gtk_main_iteration ();
    gtk_object_set_data (GTK_OBJECT (app->window), "APP_STRUCT", (gpointer) app);

    app->textviewer_xml_data = glade_xml_new (gz_get_glade_file ("textviewer.glade"), NULL);
    glade_xml_signal_autoconnect (app->textviewer_xml_data);
    app->text_viewer = text_viewer_new (app->textviewer_xml_data);

    app->overwrite_dialog = overwrite_dialog_new ((gpointer) app);
    g_list_remove (GNOME_DIALOG (app->overwrite_dialog->window)->buttons,
      g_list_nth_data (GNOME_DIALOG (app->overwrite_dialog->window)->buttons, 2)
    );

    while (gtk_events_pending ()) gtk_main_iteration ();
    app->open_filesel = gz_file_sel_new (_("Open Archive"));
    gz_file_sel_set_multi_file_select (app->open_filesel, TRUE);
    app->extract_filesel = gz_file_sel_new (_("Extract"));
    gz_file_sel_set_selection_type (app->extract_filesel,
      GZ_FILE_SEL_TYPE_DIR_ONLY);
    while (gtk_events_pending ()) gtk_main_iteration ();

    app->tooltips = gtk_tooltips_new ();

    change_menu_pixmap (app->window, "add1", GNOME_STOCK_PIXMAP_ADD);
    change_menu_pixmap (app->window, "extract1", GNOME_STOCK_PIXMAP_REMOVE);

    app->sort_menu = GTK_MENU (gtk_menu_new ());
    gtk_widget_show (GTK_WIDGET (app->sort_menu));
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (glade_lookup_widget (app->window,
      "sortby1")), GTK_WIDGET (app->sort_menu));
    tearoff = gtk_tearoff_menu_item_new ();
    gtk_widget_show (tearoff);
    gtk_menu_append (GTK_MENU (app->sort_menu), tearoff);
}

guint
main_app_get_type (void)
{
    static guint ga_type = 0;

    if (!ga_type) {
        GtkTypeInfo ga_info = {
          "MainApp",
          sizeof (MainApp),
          sizeof (MainAppClass),
          (GtkClassInitFunc) NULL,
          (GtkObjectInitFunc) main_app_init,
          (GtkArgSetFunc) NULL,
          (GtkArgGetFunc) NULL,
          (GtkClassInitFunc) NULL
        };
        ga_type = gtk_type_unique (gtk_object_get_type (), &ga_info);
    }
    return ga_type;
}


MainApp *
main_app_new (void)
{
    return MAIN_APP (gtk_type_new ((GtkType) MAIN_APP_TYPE));
}

void
main_app_destroy (MainApp *app)
{
    #define FREE_WIDGET(widget) if (app->widget != NULL) gtk_widget_destroy (app->widget)
    g_return_if_fail (app != NULL);
    g_return_if_fail (MAIN_IS_APP (app));

    gz_close_archive (app);

    FREE_WIDGET (window);
    FREE_WIDGET (clist);
    if (app->xml_data != NULL) g_free (app->xml_data);
    if (app->textviewer_xml_data != NULL) g_free (app->textviewer_xml_data);
    if (app->overwrite_dialog != NULL) overwrite_dialog_destroy (app->overwrite_dialog);
    if (app->text_viewer != NULL) text_viewer_destroy (app->text_viewer);
    g_list_free (app->sort_items);
    gtk_object_destroy (GTK_OBJECT (app->tooltips));

    winlist = g_list_remove (winlist, app);
    if (app != NULL) gtk_object_destroy (GTK_OBJECT (app));
    if (g_list_length (winlist) == 0) gtk_main_quit ();

    #undef FREE_WIDGET
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _GNOMEZIP_C_