/*
 *  GnomeZip
 *  misc.c: misc. utility functions for GnomeZip
 *
 *  Copyright (C) 2000 Hongli Lai
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _MISC_C_
#define _MISC_C_

#include <glade/glade-xml.h>
#include <config.h>
#include <gnome.h>
#include <misc.h>
#include <errno.h>
#include <sys/stat.h>
#include <unistd.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


typedef struct
{
    GMainLoop *loop;
    guint hide_signal, delete_signal, destroy_signal;
    GtkWidget *widget;
} GzLoop;


void
change_menu_pixmap (GtkWidget *awidget, gchar *name, gchar *stock)
{
    GtkWidget *widget, *pixwidget = NULL;

    widget = glade_lookup_widget (awidget, name);
    pixwidget = gnome_stock_pixmap_widget (NULL, stock);
    gtk_widget_destroy (GTK_PIXMAP_MENU_ITEM (widget)->pixmap);
    GTK_PIXMAP_MENU_ITEM (widget)->pixmap = NULL;
    gtk_widget_show (pixwidget);
    gtk_pixmap_menu_item_set_pixmap (GTK_PIXMAP_MENU_ITEM (widget),
      pixwidget);
}


gboolean
check_file (const gchar *filename)
{
    int i;
    struct stat buf;
    gchar *s;

    i = stat (filename, &buf);
    if (i == 0)
        return TRUE;
    else {
        if (g_file_test (filename, G_FILE_TEST_EXISTS) == FALSE)
        {
            s = g_strdup_printf (_("File\n'%s'\ndoes not exists."),
              filename);
            gnome_error_dialog (s);
            g_free (s);
            return FALSE;
        }

        switch (i)
        {
        case ENOENT: case ENOTDIR:
            s = g_strdup_printf (_("File\n'%s'\n does not exists."),
              filename);
            break;
        case ELOOP:
            s = g_strdup_printf (_("Too many symbolic links encountered "
              "while traversing the path of\n'%s'"),
              filename);
            break;
        case EACCES:
            s = g_strdup_printf (_("Permission to the file\n'%s'\nis denied."),
              filename);
            break;
        case ENOMEM:
            s = g_strdup_printf (_("Ran out of memory while stating file\n'%s'"),
              filename);
            break;
        case ENAMETOOLONG:
            s = g_strdup_printf (_("The filename\n'%s'\nis too long"),
              filename);
            break;
        default:
            s = g_strdup_printf (_("An unknown error occured when stating "
              "file\n'%s'"), filename);
            break;
        }
        gnome_error_dialog (s);
        g_free (s);
        return FALSE;
    }
}


gchar *
g_concat_path (const gchar *filename, gchar *path)
{
    if (filename == NULL && path == NULL)
        return NULL;

    if (path != NULL && path[strlen (path) - 1] == '/' && filename != NULL)
        return g_strdup_printf ("%s%s", path, filename);
    else if (path != NULL && path[strlen (path) - 1] != '/' && filename != NULL)
        return g_strdup_printf ("%s/%s", path, filename);
    else if (path == NULL && filename != NULL)
        return g_strdup (filename);
    else if (path != NULL && path[strlen (path) - 1] == '/' && filename == NULL)
        return g_strndup (path, strlen (path) - 1);
    else if (path != NULL && path[strlen (path) - 1] != '/' && filename == NULL)
        return g_strdup (path);
    else
        return NULL;
}


GtkWidget *
glade_lookup_widget (GtkWidget *widget, const gchar *widget_name)
{
	GtkWidget *found_widget, *parent;
	GladeXML *xml = NULL;

	while (xml == NULL)
	{
		/* the following line is to allow to override the
		 * GladeXML object or to set it to an object which was
		 * not created by libglade. Useful for popup menus */
		xml = gtk_object_get_data (GTK_OBJECT(widget), "widget_tree");
		if (!xml) xml = glade_get_widget_tree(widget);

		if (GTK_IS_MENU(widget))
			parent = gtk_menu_get_attach_widget (GTK_MENU(widget));
		else
			parent = widget->parent;

		if (parent == NULL) 
			break;

		widget = parent;
	}

	found_widget = glade_xml_get_widget(xml, widget_name);

	if (!found_widget)
		g_warning ("Widget not found: %s", widget_name);
 
	return found_widget;
}


gint
gtk_clist_get_selected (GtkCList *clist)
{
    gint i = -1;

    for (i = 0; i < g_list_length (GTK_CLIST (clist)->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (GTK_CLIST (clist)->row_list, i);
        if (row->state == GTK_STATE_SELECTED)
            return i;
    }
    return i;
}


GList *
gtk_clist_get_selections (GtkCList *clist)
{
    GList *result = NULL;
    guint i;

    for (i = 0; i < g_list_length (GTK_CLIST (clist)->row_list); i++)
    {
        GtkCListRow *row = g_list_nth_data (GTK_CLIST (clist)->row_list, i);
        if (row->state == GTK_STATE_SELECTED)
            result = g_list_append (result, row);
    }
    return result;
}


static void
gtk_widget_run_disconnect_signals (GzLoop *loop)
{
    gtk_signal_disconnect (GTK_OBJECT (loop->widget), loop->hide_signal);
    gtk_signal_disconnect (GTK_OBJECT (loop->widget), loop->delete_signal);
    gtk_signal_disconnect (GTK_OBJECT (loop->widget), loop->destroy_signal);
    g_main_quit (loop->loop);
}


void
gtk_widget_run (GtkWidget *widget)
{
    GzLoop *loop;

    loop = (GzLoop *) g_new0 (GzLoop, 1);
    loop->loop = g_main_new (FALSE);
    loop->widget = widget;
    loop->hide_signal = gtk_signal_connect_object (GTK_OBJECT (widget), "hide",
      GTK_SIGNAL_FUNC (gtk_widget_run_disconnect_signals), (GtkObject *) loop);
    loop->delete_signal = gtk_signal_connect_object (GTK_OBJECT (widget), "delete_event",
      GTK_SIGNAL_FUNC (gtk_widget_run_disconnect_signals), (GtkObject *) loop);
    loop->destroy_signal = gtk_signal_connect_object (GTK_OBJECT (widget), "destroy",
      GTK_SIGNAL_FUNC (gtk_widget_run_disconnect_signals), (GtkObject *) loop);

    gtk_widget_show (widget);
    g_main_run (loop->loop);
    g_main_destroy (loop->loop);
    g_free (loop);
}


void
gz_close_parent_window (GtkWidget *widget, const gchar *parent_window_name)
{
    gtk_widget_destroy (glade_lookup_widget (widget, parent_window_name));
}


gchar *
gz_get_glade_file (const gchar *filename)
{
    gchar *files[] = {
        g_strconcat ("../ui/", filename, NULL),
        g_strconcat ("ui/", filename, NULL),
        g_strconcat ("./", filename, NULL),
        g_strconcat (PACKAGE_GLADE_DIR, filename, NULL),
        NULL
    };
    gchar *fn = NULL;
    guint i = 0;

    while (files[i] != NULL)
    {
        if (g_file_test (files[i], G_FILE_TEST_EXISTS) == TRUE)
        {
            fn = files[i];
            break;
        }
        i++;
    }

    fn = g_strdup (fn);
    while (files[i] != NULL)
    {
        g_free (files[i]);
        i++;
    }
g_print ("%s\n", fn);
    if (fn == NULL)
    {
        gnome_error_dialog (g_strdup_printf (_("Error: cannot find file '%s'"), files[i - 1]));
        g_print (_("Error: cannot find file '%s'\n"), files[i - 1]);
    }
    return fn;
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _MISC_C_