/*
 *  GnomeZip
 *  overwrite-dialog.c - Overwrite Dialog
 *
 *  Copyright (c) 2000 Hongli Lai <hongli@telekabel.nl>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _OVERWRITE_DIALOG_C_
#define _OVERWRITE_DIALOG_C_

#include <gnome.h>
#include <overwrite-dialog.h>
#include <text-viewer.h>
#include <misc.h>
#include <libgarp/garp-archive.h>
#include <gnomezip.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


static void
overwrite_dialog_init (OverwriteDialog *obj, MainApp *app)
{
    GtkWidget *menu;

    obj->xml_data = app->xml_data;

    obj->window = glade_xml_get_widget (obj->xml_data, "overwrite");
    gtk_object_set_data (GTK_OBJECT (obj->window), "APP_STRUCT",
      (gpointer) obj);

    obj->clist = glade_lookup_widget (obj->window, "clist1");

    obj->text_viewer = app->text_viewer;

    menu = gtk_option_menu_get_menu (GTK_OPTION_MENU (glade_lookup_widget (
      obj->window, "optionmenu1")));
    gtk_signal_connect (GTK_OBJECT (menu), "selection-done",
      GTK_SIGNAL_FUNC (overwrite_dialog_change_selection_mode),
      glade_lookup_widget (obj->window, "optionmenu1"));
}


guint
overwrite_dialog_get_type (void)
{
    static guint ga_type = 0;

    if (!ga_type) {
        GtkTypeInfo ga_info = {
          "OverwriteDialog",
          sizeof (OverwriteDialog),
          sizeof (OverwriteDialogClass),
          (GtkClassInitFunc) NULL,
          (GtkObjectInitFunc) NULL,
          (GtkArgSetFunc) NULL,
          (GtkArgGetFunc) NULL,
          (GtkClassInitFunc) NULL
        };
        ga_type = gtk_type_unique (gtk_object_get_type (), &ga_info);
    }
    return ga_type;
}


OverwriteDialog *
overwrite_dialog_new (gpointer *app)
{
    OverwriteDialog *dlg;

    dlg = OVERWRITE_DIALOG (gtk_type_new ((GtkType) OVERWRITE_DIALOG_TYPE));
    overwrite_dialog_init (dlg, (MainApp *) app);
    return dlg;
}


void
overwrite_dialog_change_selection_mode (GtkButton *button, gpointer user_data)
{
    if (g_strcasecmp (GTK_LABEL (GTK_BIN (user_data)->child)->label,
       "Extended") == 0)
        gtk_clist_set_selection_mode (GTK_CLIST (glade_lookup_widget (
          GTK_WIDGET (button), "clist1")), GTK_SELECTION_EXTENDED);
    else
        gtk_clist_set_selection_mode (GTK_CLIST (glade_lookup_widget (
          GTK_WIDGET (button), "clist1")), GTK_SELECTION_MULTIPLE);
}


void
overwrite_dialog_destroy (OverwriteDialog *obj)
{
    g_return_if_fail (obj != NULL);
    g_return_if_fail (OVERWRITE_IS_DIALOG (obj) == TRUE);

    if (obj->window != NULL)      gtk_widget_destroy (obj->window);
    gtk_object_destroy (GTK_OBJECT (obj));
}


gboolean
overwrite_dialog_on_delete (GtkWidget *widget, gpointer user_data)
{
    gtk_button_clicked (GTK_BUTTON (g_list_nth_data (GNOME_DIALOG (
      widget)->buttons, 1)));
    return TRUE;
}


gboolean
overwrite_dialog_on_destroy (GtkWidget *widget, gpointer user_data)
{
    overwrite_dialog_destroy (OVERWRITE_DIALOG (gtk_object_get_data (
      GTK_OBJECT (widget), "APP_STRUCT")));
    return TRUE;
}


void
overwrite_dialog_view_file (GtkWidget *button, gpointer user_data)
{
    OverwriteDialog *dialog;
    TextViewer *viewer;
    GarpArchiveEntry *entry;
    GList *list;
    gchar *s;

    dialog = OVERWRITE_DIALOG (gtk_object_get_data (GTK_OBJECT (
      glade_lookup_widget (button, "overwrite")), "APP_STRUCT"));
    viewer = dialog->text_viewer;
    gtk_widget_show (viewer->window);

    list = gtk_clist_get_selections (GTK_CLIST (glade_lookup_widget (
      button, "clist1")));
    entry = gtk_clist_get_row_data (GTK_CLIST (glade_lookup_widget (
      button, "clist1")),
      gtk_clist_get_selected (GTK_CLIST (glade_lookup_widget (
      button, "clist1")))
    );

    if (entry->pathname)
        s = g_strdup_printf ("%s%s/%s", dialog->extract_to_dir,
          entry->pathname, entry->filename);
    else
        s = g_strdup_printf ("%s%s", dialog->extract_to_dir, entry->filename);
    text_viewer_open_file (viewer, s);
    g_list_free (list);
    g_free (s);
}


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif _OVERWRITE_DIALOG_C_
